// TestCaseGeneratorUI
// (c)2007 Mark Bulmahn. mBulmahn@web.de
// License: GPL 2.0
// Project: TestCaseGenerator. http://www.codeplex.com/TestCaseGenerator

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace TestCaseGenerator.Controls
{
    public partial class ValueList : UserControl
    {
        public delegate void ItemAddedEventHandler(string inputText);
        public delegate void ItemSelectionChangedHandler(object sender, EventArgs e);
        public delegate void ItemRemovedHandler(string text);
        public event ItemSelectionChangedHandler ItemSelectionChanged;
        public event ItemAddedEventHandler ItemAdded;
        public event ItemRemovedHandler ItemRemoved;

        string buttonText = "Input";
        public string ButtonText
        {
            get { return buttonText; }
            set
            {
                buttonText = value;
                butAddVal.Text = buttonText;
            }
        }

        bool allowOnlyUniqueValues = false;
        public bool AllowOnlyUniqueValues
        {
            get { return allowOnlyUniqueValues; }
            set { allowOnlyUniqueValues = value; }
        }

        public ListView ItemList
        {
            get { return listVal; }
            set { listVal = value; }
        }

        public ValueList()
        {
            InitializeComponent();
            wertlschenToolStripMenuItem.Enabled = false;
        }

        private void butAddVal_Click(object sender, EventArgs e)
        {
            bool valid = true;
            if (textInput.Text.Trim() != "")
            {
                if (allowOnlyUniqueValues)
                {
                    foreach (ListViewItem lvi in listVal.Items)
                    {
                        if (lvi.Text == textInput.Text)
                        {
                            valid = false;
                            break;
                        }
                    }
                }
                if (valid)
                {
                    listVal.Items.Add(textInput.Text);
                    if (ItemAdded != null)
                        ItemAdded(textInput.Text);
                }
                else
                {
                    MessageBox.Show("Wert existiert bereits in der Liste");
                }
                textInput.Text = "";
            }
            else
            {
                MessageBox.Show("Leere Werte sind nicht erlaubt!");
            }
            textInput.Focus();
        }

        private void ValueList_Load(object sender, EventArgs e)
        {
            listVal.Columns[0].AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize);
            butAddVal.Text = buttonText;
        }

        public void FillList(List<string> list)
        {
            listVal.Items.Clear();
            foreach (string s in list)
                listVal.Items.Add(s);
        }

        private void DeleteItem()
        {
            if (listVal.SelectedItems.Count > 0)
            {
                string text = listVal.SelectedItems[0].Text;
                listVal.Items.Remove(listVal.SelectedItems[0]);
                if (ItemRemoved != null)
                    ItemRemoved(text);
            }
        }

        private void listVal_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listVal.SelectedItems.Count > 0)
                wertlschenToolStripMenuItem.Enabled = true;
            else
                wertlschenToolStripMenuItem.Enabled = false;
            if (ItemSelectionChanged != null)
            {
                ItemSelectionChanged(sender, e);
            }
        }

        private void wertlschenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DeleteItem();
        }
    }
}
