// PictWrapper
// (c)2007 Mark Bulmahn. mBulmahn@web.de
// License: GPL 2.0
// Project: TestCaseGenerator. http://www.codeplex.com/TestCaseGenerator

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;
using ConfigMgmtEx;

namespace PICTWrapper
{
    /// <summary>
    /// This class is for process handling stuff using the PICT.EXE inside this
    /// project.
    /// The tool can be downloaded from http://download.microsoft.com/download/f/5/5/f55484df-8494-48fa-8dbd-8c6f76cc014b/pict33.msi
    /// It was written by Jacek Czerwonka for Microsoft Corp. So this tool is not included with the project.
    /// It should be downloaded from the above link and included in the Components directory of this project.
    /// Don't forget to activate the "Copy always" option for it.
    /// </summary>
    public class PictWrapper
    {
        public static string[] DoPict(List<string> modelFile, int orderOfCombinations)
        {
            string tempSourceFn = Path.GetTempPath() + "pictmodel.txt";

            // write temporary file
            File.WriteAllLines(tempSourceFn, modelFile.ToArray());

            // create cases with pict
            ProcessStartInfo pi = CreateStartInfo(tempSourceFn, orderOfCombinations);
            string o = ExecutePict(pi);

            // create string array from stdout
            string[] lines = o.Split('\n');
            for(int i = 0; i < lines.Length; i++)
                lines[i] = lines[i].Replace("\r", "");
            return lines;
        }

        private static string ExecutePict(ProcessStartInfo pi)
        {
            Process pict = new Process();
            pict.StartInfo = pi;
            pict.Start();
            StreamReader sr = pict.StandardOutput;
            string o = sr.ReadToEnd();
            return o;
        }

        private static ProcessStartInfo CreateStartInfo(string tempSourceFn, int order)
        {
            // ToDo: Konfig einfgen
            string cmd = AppDomain.CurrentDomain.BaseDirectory + "pict.exe";
            string opt = "\"" + tempSourceFn + "\"" + " /o:" + order.ToString();

            File.AppendAllText("dbg.txt", cmd);
            File.AppendAllText("dbg.txt", opt);
            ProcessStartInfo pi = new ProcessStartInfo(cmd, opt);
            pi.UseShellExecute = false;
            pi.WindowStyle = ProcessWindowStyle.Hidden;
            pi.CreateNoWindow = true;
            pi.RedirectStandardOutput = true;
            return pi;
        }
    }
}
